class NewString(str):
    """
    New string class that inherits all attributes and methods of the object str
    """

    def multireplace(self, replacement_sumbols, new_sumbols=''):
        """
        Method for replacing specified characters in a string with new characters.
        Single replacement and new characters are objects that are
        single words, parts of words, letters, special characters,
        or combinations thereof, enclosed in string literals.

        There are three ways to specify replacement and new characters:
        1)  Replaced and new characters are grouped into lists or tuples of the same length.
            In this case, each character being replaced is changed to
            a new character located in the same place (having the same index).

        2)  One replacement character and one new character specified.

        3)  One new character and multiple replacement characters are specified,
            grouped into a list or tuple.
            In this case, each character being replaced is changed to the same new character.

        When setting parameters in other ways than the above three, an valueError exception is thrown.
        All values contained in the replacement_sumbols and new_sumbols will be converted to strings.

        :param replacement_sumbols: str, list, tuple,
        :param new_sumbols: str, list, tuple
        :return: str
        """

        # verification stage
        if isinstance(replacement_sumbols, (list, tuple)):
            if isinstance(new_sumbols, (list, tuple)):
                if len(replacement_sumbols) != len(new_sumbols):
                    raise ValueError('different number of elements replacement_sumbols and new_sumbols')
            elif not isinstance(new_sumbols, str):
                raise ValueError('wrong type new_sumbols')
        elif isinstance(replacement_sumbols, str):
            if not isinstance(new_sumbols, str):
                raise ValueError('wrong type new_sumbols')
        else:
            raise ValueError('wrong type replacement_sumbols')

        # replacement stage
        index = 0
        if isinstance(replacement_sumbols, (list, tuple)):
            replacement_sumbols = [str(i) for i in replacement_sumbols]
            while index < len(self):
                letter = self[index]
                for position, sumbol in enumerate(replacement_sumbols):
                    if letter == sumbol[0]:
                        if self[index:index + len(sumbol)] == sumbol:
                            if isinstance(new_sumbols, str):
                                new_sumbol = new_sumbols
                            else:
                                new_sumbol = str(new_sumbols[position])
                            self = self[:index] + new_sumbol + self[index + len(sumbol):]
                            index += len(new_sumbol) - 1
                            break
                index += 1
        else:
            while index < len(self):
                letter = self[index]
                if letter != replacement_sumbols[0]:
                    index += 1
                    continue
                if self[index: index + len(replacement_sumbols)] == replacement_sumbols:
                    self = self[:index] + new_sumbols + self[index + len(replacement_sumbols):]
                    index += len(new_sumbols)
                    continue
                index += 1
        return self


import unittest

from multireplace import NewString


class TestStrMetod(unittest.TestCase):

    def test_errors(self):
        new_str = NewString('qwerty')
        init_params = [
            ((i for i in range(3)), ''),            # generator and str
            (print, ''),                            # function and str
            ('', print),                            # str and function
            ('', (i for i in range(3))),            # str and generator
            (['q', 'w', 'e'], ('a', 'b')),          # different number
            ('', {'q'}),                            # wrong type new_sumbols
        ]
        for param in init_params:
            self.assertRaises(ValueError, new_str.multireplace, *param)

    def test_functionality(self):
        new_str = NewString('qwerty 351')
        init_params = [
            (['w', 'r'], ''),
            ('ert', ''),
            (['q', 'ty'], ('b', 'q')),
            (('1', 'y'), ['2', 'q']),
            ([3, 2, 5], (10, 11, 12))
        ]

        results = [
            'qety 351',
            'qwy 351',
            'bwerq 351',
            'qwertq 352',
            'qwerty 10121'
        ]

        for sumbols, result in zip(init_params, results):
            replacement_sumbols, new_sumbols = sumbols
            self.assertEqual(
                new_str.multireplace(replacement_sumbols=replacement_sumbols, new_sumbols=new_sumbols),
                result
            )

    def test_working_string_metods(self):
        new_str = NewString('qwerty')

        self.assertEqual(new_str.replace('q', ''), 'werty')
        self.assertEqual(new_str.split('e'), ['qw', 'rty'])
        self.assertEqual(new_str[2], 'e')
        self.assertEqual(new_str[1:3], 'we')
        self.assertEqual(new_str.upper(), 'QWERTY')
        self.assertEqual(new_str.title(), 'Qwerty')
        self.assertEqual(new_str.endswith('ty'), True)
        self.assertEqual(new_str.startswith('qu'), False)






from setuptools import setup, find_packages

with open('README.md') as file:
    readme = file.read()

setup(
    name='multireplace',
    version='0.0.2',
    author='Oshten',
    description='Method for multi replace in strings',
    long_description=readme,
    long_description_content_type='text/markdown',
    url='https://github.com/Oshten/multireplace',
    packages=find_packages(),
    install_requires=[],
    classifiers=[
        "Programming Language :: Python :: 3.7",
        "License :: OSI Approved :: GNU General Public License v3 (GPLv3)",
    ],
)